==> PrettyQt-0.177.1.tar.gz: PrettyQt-0.177.1/prettyqt/prettyqtest/exceptions.py <==
from __future__ import annotations

from contextlib import contextmanager
import functools
import sys
import traceback

import pytest


@contextmanager
def capture_exceptions():
    """Context manager that captures exceptions that happen insides its context.

    Returns them as a list of (type, value, traceback) after the context ends.
    """
    manager = _QtExceptionCaptureManager()
    manager.start()
    try:
        yield manager.exceptions
    finally:
        manager.finish()


def _except_hook(type_, value, tback, exceptions=None):
    """Hook functions installed by _QtExceptionCaptureManager."""
    exceptions.append((type_, value, tback))
    sys.stderr.write(format_captured_exceptions([(type_, value, tback)]))


class _QtExceptionCaptureManager:
    """Manages exception capture context."""

    def __init__(self):
        self.old_hook = None
        self.exceptions = []

    def start(self):
        """Start exception capturing by installing a hook into sys.excepthook.

        Record exceptions received into ``self.exceptions``.
        """
        self.old_hook = sys.excepthook
        sys.excepthook = functools.partial(_except_hook, exceptions=self.exceptions)

    def finish(self):
        """Stop exception capturing, restoring the original hook.

        Can be called multiple times.
        """
        if self.old_hook is not None:
            sys.excepthook = self.old_hook
            self.old_hook = None

    def fail_if_exceptions_occurred(self, when):
        """Call pytest.fail() with a message if exceptions have been captured so far.

        Before pytest.fail() is called, also finish capturing.
        """
        if self.exceptions:
            self.finish()
            exceptions = self.exceptions
            self.exceptions = []
            prefix = "%s ERROR: " % when
            msg = prefix + format_captured_exceptions(exceptions)
            del exceptions[:]  # Don't keep exceptions alive longer.
            pytest.fail(msg, pytrace=False)


def format_captured_exceptions(exceptions) -> str:
    """Format exceptions given as (type, value, traceback) into a string.

    Suitable to display as a test failure.
    """
    from io import StringIO

    stream = StringIO()
    stream.write("Exceptions caught in Qt event loop:\n")
    sep = "_" * 80 + "\n"
    stream.write(sep)
    for (exc_type, value, tback) in exceptions:
        traceback.print_exception(exc_type, value, tback, file=stream)
        stream.write(sep)
    return stream.getvalue()


def _is_exception_capture_enabled(item):
    """Return if exception capture is disabled for the given test item."""
    disabled = item.get_closest_marker("qt_no_exception_capture") or item.config.getini(
        "qt_no_exception_capture"
    )
    return not disabled


class TimeoutError(Exception):
    """Exception thrown by :class:`pytestqt.qtbot.QtBot` methods."""

    pass


# backward compatibility alias
SignalTimeoutError = TimeoutError

==> PrettyQt-0.183.4.tar.gz: PrettyQt-0.183.4/prettyqt/prettyqtest/exceptions.py <==
from __future__ import annotations

from contextlib import contextmanager
import functools
import sys
import traceback

import pytest


@contextmanager
def capture_exceptions():
    """Context manager that captures exceptions that happen insides its context.

    Returns them as a list of (type, value, traceback) after the context ends.
    """
    manager = _QtExceptionCaptureManager()
    manager.start()
    try:
        yield manager.exceptions
    finally:
        manager.finish()


def _except_hook(type_, value, tback, exceptions=None):
    """Hook functions installed by _QtExceptionCaptureManager."""
    exceptions.append((type_, value, tback))
    sys.stderr.write(format_captured_exceptions([(type_, value, tback)]))


class _QtExceptionCaptureManager:
    """Manages exception capture context."""

    def __init__(self):
        self.old_hook = None
        self.exceptions = []

    def start(self):
        """Start exception capturing by installing a hook into sys.excepthook.

        Record exceptions received into ``self.exceptions``.
        """
        self.old_hook = sys.excepthook
        sys.excepthook = functools.partial(_except_hook, exceptions=self.exceptions)

    def finish(self):
        """Stop exception capturing, restoring the original hook.

        Can be called multiple times.
        """
        if self.old_hook is not None:
            sys.excepthook = self.old_hook
            self.old_hook = None

    def fail_if_exceptions_occurred(self, when):
        """Call pytest.fail() with a message if exceptions have been captured so far.

        Before pytest.fail() is called, also finish capturing.
        """
        if self.exceptions:
            self.finish()
            exceptions = self.exceptions
            self.exceptions = []
            prefix = f"{when} ERROR: "
            msg = prefix + format_captured_exceptions(exceptions)
            del exceptions[:]  # Don't keep exceptions alive longer.
            pytest.fail(msg, pytrace=False)


def format_captured_exceptions(exceptions) -> str:
    """Format exceptions given as (type, value, traceback) into a string.

    Suitable to display as a test failure.
    """
    from io import StringIO

    stream = StringIO()
    stream.write("Exceptions caught in Qt event loop:\n")
    sep = "_" * 80 + "\n"
    stream.write(sep)
    for (exc_type, value, tback) in exceptions:
        traceback.print_exception(exc_type, value, tback, file=stream)
        stream.write(sep)
    return stream.getvalue()


def _is_exception_capture_enabled(item):
    """Return if exception capture is disabled for the given test item."""
    disabled = item.get_closest_marker("qt_no_exception_capture") or item.config.getini(
        "qt_no_exception_capture"
    )
    return not disabled
